#ifndef ICLIENT_HH
#define ICLIENT_HH

#include <sys/epoll.h>
#include <netinet/tcp.h>
#include <netdb.h> 

#include "imsg.hh"
#include "message.hh"
#include "msgConstants.hh"

class iclient {

	private:

	    int sock, port, epollfdr;
	    struct sockaddr_in serv_addr;
	    struct hostent *server;
		struct epoll_event ev;
		struct epoll_event events[1];
	
	public:

		iclient(){}

		bool connect(const char *s, int p) {
		    port = p;
		    sock = socket(AF_INET, SOCK_STREAM, 0);
		    if (sock < 0)  return false;
			int notcpdelay = 1;
			if(setsockopt(sock, SOL_TCP, TCP_NODELAY, &notcpdelay, sizeof(int)) == -1) ; //"TCP_NODELAY unavailable";   
		    server = gethostbyname(s);
		    bzero((char *) &serv_addr, sizeof(serv_addr));
		    serv_addr.sin_family = AF_INET;
		    bcopy((char *)server->h_addr, (char *)&serv_addr.sin_addr.s_addr, server->h_length);
		    serv_addr.sin_port = htons(port);
		    if (::connect(sock,(struct sockaddr *) &serv_addr,sizeof(serv_addr)) < 0) return false;
			epollfdr = epoll_create(1);
			if(!epollfdr) return false;
			ev.events = EPOLLIN;
			ev.data.fd = sock;
			if(epoll_ctl(epollfdr, EPOLL_CTL_ADD, sock, &ev) < 0) return false;
        	return true;
		}

		void subscribe(const char *name) {
			message post;
			post.encode(DEFAULT_SERVER_NAME, name);
			send(post);
		}
		
		void disconnect() {
			close(sock);		
		}
	
		bool checkReadSocket(unsigned int timeout) {
			int nfds = epoll_wait(epollfdr, events, 1, timeout);
			if(nfds <= 0) return false;
			return true;
		}

		bool send(message data) {
			imsg msg;
			msg.encode(data.data());
			long n;
			size_t tot = 0;
			size_t bleft = msg.length();
			while(tot < msg.length()) {
				n = ::send(sock, msg.data() + tot, bleft, MSG_NOSIGNAL);
				if(n < 1) break;
				tot += n;
				bleft -= tot;
			}
			return (tot == msg.length());
		}

		message receive(int &error) {
			message result;
			imsg msg; ssize_t res;
			char header[msg.hsize()];
			res = ::recv(sock, header, msg.hsize(), 0);
			if(res != static_cast<ssize_t>(msg.hsize())) error = 1;
			msg.decodeHeader(header);
			char body[msg.blength()];
			res = ::recv(sock, body, msg.blength(), MSG_WAITALL);
			if(res != static_cast<ssize_t>(msg.blength())) error = 1;
			msg.decodeBody(body);
			result.decode(msg.body());
			return result;			
		}
		
};

#endif
