#include "msgServer.hh"

using namespace std;

msgServer::msgServer(unsigned int p, unsigned int inact) {
	this->currentPort = p;
	this->inactivity_time = inact;
	this->isRunning = iserv.createServer(this->currentPort, MAX_CLIENT_CONNECTIONS);
	isMessagesWaiting = false;
	if(this->isRunning) {
		iserv.setInactivityTime(this->inactivity_time);
		cout << "----------------------" << endl;
		cout << "msgconnector Server in now running on port localhost:" << this->currentPort << endl;
		cout << " - max authorized connections: " << MAX_CLIENT_CONNECTIONS << endl;
		cout << " - max inactivity time: " << this->inactivity_time << endl;
		cout << "----------------------" << endl;		
	}
}

void msgServer::setInactivityTime(unsigned int inact) {
	this->inactivity_time = inact;
	if(this->isRunning) {
		iserv.setInactivityTime(this->inactivity_time);
	}
}

void msgServer::addConnection(int clientId, string clientName) {
	this->clientsById.insert(make_pair(clientId, clientName));
	this->clientsByName.insert(make_pair(clientName, clientId));
	cout << clientName << " is now connected" << endl;
}

void msgServer::removeConnection(int c) {
	if(this->clientsById.find(c) != this->clientsById.end()) {
		string n = this->clientsById.find(c)->second;
		this->clientsById.erase(this->clientsById.find(c));
		this->clientsByName.erase(this->clientsByName.find(n));
		cout << n << " is disconnected" << endl;
	}
	iserv.removeClientConnection(c);
}

void msgServer::manageIncomingMessage(int c, message m) {
	string dest = m.getStringHead();
	string content = m.getStringBody();
	if(dest == DEFAULT_SERVER_NAME) { // command
		if(strncmp(content.c_str(), REQUEST_TEST_CONNECTED, strlen(REQUEST_TEST_CONNECTED)) == 0) {
			size_t lwho = strlen(content.c_str()) - strlen(REQUEST_TEST_CONNECTED) -1;
			char who[lwho+1];
			memcpy(who, content.c_str()+strlen(REQUEST_TEST_CONNECTED)+1, lwho+1);
			bool cwho = (this->clientsByName.find(string(who)) != this->clientsByName.end());
			message res;
			if(cwho) res.encode(DEFAULT_SERVER_NAME, REQUEST_TRUE);
			else res.encode(DEFAULT_SERVER_NAME, REQUEST_FALSE);
			iserv.send(c, res);
		}
		else this->addConnection(c, content);
	}
	else { // transfer
		if(this->clientsByName.find(dest) != this->clientsByName.end()) {
			string from = this->clientsById.find(c)->second;
			int client = this->clientsByName.find(dest)->second;
			message temp;
			temp.encode(from.c_str(), content.c_str());
			if(!iserv.send(client, temp)) cout << "error in send" << endl;
		}
	}
}

void msgServer::run() {
	while(this->isRunning) {
		vector<pair<int, int> > events = iserv.waitReadEvents();
		for(int i = 0 ; i < events.size() ; i++) {
			pair<int, int> event = events[i];
			if(event.first = iserv.READ_EVENT && event.second > 0) {
				if(iserv.isMainSocket(event.second)) iserv.addClientConnection(); 
				else {
					if(event.first == iserv.ERROR_EVENT) this->removeConnection(event.second);
					else { 
						pair<int, message> recpt = iserv.receive(event.second);
						if(recpt.first == iserv.STAT_ERR) this->removeConnection(event.second);
						else this->manageIncomingMessage(event.second, recpt.second);
					}
				}
			}
			if(event.second == 0) this->shutdown();
		}
	}
}

void msgServer::shutdown() {
	if(this->isRunning) {
		this->isRunning = false;
		iserv.stopServer();
	}
}

msgServer::~msgServer() {
	cout << "shuting down server... " << endl;
	this->shutdown();
}

